@@ -17,8 +17,10 @@ except ImportError:
 
 
 class TranslateJson(object):
-    def __init__(self, js_file, output_md, output_gitbook):
+    def __init__(self, js_file, link_path, dir_type, output_md, output_gitbook):
         self.js_file = js_file
+        self.link_path = link_path
+        self.dir_type = dir_type
         self.output_md = output_md
         self.output_gitbook = output_gitbook
 
@@ -32,6 +34,10 @@ class TranslateJson(object):
         self.history_group = {}
         self.pre_group = ''
         self.all_reg_data = ''
+        self.all_reg_list = []
+        self.reg_data = ''
+        self.path_addr_dict = {}
+        self.max_len = ''
 
     def get_dpath(self, in_dict, pre_path = ''):
         path_list = []
@@ -119,7 +125,7 @@ class TranslateJson(object):
             else:
                 next_num = self.history_group[path]['depth'] * \
                         (self.history_group[path]['num'] + next_num -1)
-        self.history_addrr = self.history_group[path]['start'] + next_num * (0x0004)
+        self.history_addrr = self.history_group[path]['start'] + (next_num-1) * (0x0004)
         self.history_offset = self.history_addrr
     
         if len(self.enter_group_list) > 0:
@@ -130,7 +136,7 @@ class TranslateJson(object):
         base_changed = 0
         block_path = dpath.split("registers")[0]
         base_addr = int(self.get_delem(self.js_dict, block_path, key='base_addr'), 16)
-        if self.history_base == 0:
+        if self.history_base == -1:
             self.history_base = base_addr
             self.history_offset = base_addr
             self.history_addrr = base_addr
@@ -280,34 +286,66 @@ class TranslateJson(object):
         return re_fields
     
     
-    def show_group(self, lm, group, name, des, st_addr):
+    def show_group(self, lm, group, name, des, st_addr, hex_reset):
         for i in range(int(group['range'])):
             new_name = name + "||" + str(i)
             new_addr = st_addr + i * group['step']
             if group['group'] != []:
                 for g in group['group']:
-                    self.show_group(lm, g, new_name, des, new_addr)
+                    self.show_group(lm, g, new_name, des, new_addr, hex_reset)
             else:
-                self.all_reg_data += lm.md_create_body_line([new_name, hex(new_addr), des], self.output_md)
+                self.all_reg_list.append({
+                    'name':new_name,
+                    'addr':self.zero_fill(hex(new_addr)),
+                    'reset':hex_reset,
+                    'des':des,
+                    })
+
+    def add_reset(self, reset, index_range, value):
+        scope = index_range.strip("[]").split(":")
+        end = int(scope[0]) + 1
+        start = int(scope[-1])
+        if int(value, 16) == 0:
+            pass
+        else:
+            fill_list = list(bin(int(value, 16))[2:])
+            for i in range(start, end):
+                if fill_list != []:
+                    reset[i] = fill_list.pop(-1)
+                else:
+                    break
+
+    def get_reset(self, reset):
+        value = ''
+        for i in range(32):
+            value += str(reset[31-i])
+        tmp = hex(int(value, 2))[2:]
+        #if tmp == '0':
+        #    hex_value = "0x" + tmp
+        #else:
+        hex_value = "0x" + tmp.zfill(8)
+        return hex_value
+
+    def zero_fill(self, in_hex):
+        new_hex = "0x" + in_hex[2:].zfill(self.max_len)
+        return new_hex
     
     def create_md3(self, dpath, lm):
         ls = libsvg.LibSVG()
-        #print(dpath)
+        reset = {}
+        for k,v in enumerate("0"*32):
+            reset[k] = v
     
         is_group = 0
         register = self.get_delem(self.js_dict, dpath)
         name = self.get_delem(self.js_dict, dpath, key = 'RegName' )
         name = name.upper()
-        addr = self.get_addr(dpath)
+        #addr = self.get_addr(dpath)
+        addr = self.zero_fill(self.path_addr_dict[dpath])
         if dpath.find("group_reg") >= 0:
             is_group = 1
             addr_msg, scope_msg, group_msg = self.get_groupmsg(dpath)
-            #group_list = self.get_group_path(dpath)
-            #for i in group_list:
-            #    gname = self.get_delem(self.js_dict, i, key="RegName")
-            #    gname = gname.upper()
-            #    name = gname + "@" + name
-            addr = str(addr) + addr_msg + ")</br>(" + scope_msg
+            addr = str(addr) + addr_msg + "(" + scope_msg + " )"
         #print(name, "(" + addr + ")")
     
         #if name.endswith("reserved"):
@@ -315,12 +353,6 @@ class TranslateJson(object):
             return -1
     
         des = self.get_delem(self.js_dict, dpath, key = 'Description')
-        if is_group:
-            st_addr = int(addr.split("+")[0], 16)
-            self.show_group(lm, group_msg, name ,des, st_addr)
-        else:
-            st_addr = addr
-            self.all_reg_data += lm.md_create_body_line([name, st_addr, des], self.output_md)
         fields = self.get_delem(self.js_dict, dpath, key = 'Fields')
         reserve_fields = self.get_reserve_fields(fields)
         fields.extend(reserve_fields)
@@ -329,98 +361,172 @@ class TranslateJson(object):
             ls.add_line(table, f['range'])
         for f in fields:
             ls.add_reset_value(table, f['range'], f['reset_value'], f['name'])
+            self.add_reset(reset, f['range'], f['reset_value'])
         for f in fields:
             ls.add_name(table, f['range'], f['name'])
+        hex_reset = self.get_reset(reset)
+        if is_group:
+            st_addr = int(addr.split("+")[0], 16)
+            self.show_group(lm, group_msg, name ,des, st_addr, hex_reset)
+        else:
+            st_addr = addr
+            self.all_reg_list.append({
+                'name':name,
+                'addr':st_addr,
+                'reset':hex_reset,
+                'des':des,
+                })
         #ls.write_down(table)
         ls.indent(table)
         svg_data = ls.svg_transform(table)
         svg_name = name + ".svg"
         with open(svg_name, "w") as fd:
             fd.write(svg_data)
-        png_name = name + ".png"
-        cairosvg.svg2png(url=svg_name, write_to = png_name)
-        os.system("rm " + svg_name)
-    
-        #register_data = "![](" + svg_name + ")\n<br>"
-        register_data = "![](" + png_name + ")\n<br>"
-        for f in fields:
+        register_data = "![](" + svg_name + ")\n<br>"
+
+        fields_len = len(fields)
+        for i in range(fields_len):
+            f = fields[fields_len - i -1]
             if f['name'] != 'reserved':
-                line = self.add_description(f['name'],f['description'])
+                line = self.add_description(f['name'],f['description'], f['access'])
                 register_data += line + "\n<br>"
     
-    
-        #table_title = 'Register:' + name.split("@")[-1] + ' ('  + addr + ')'
-        table_title = 'Register:' + name + ' ('  + addr + ')'
-        #print(table_title)
-        #register_data = "## " + '<center>' + table_title + '</center>\n\n' +\
-        register_data = "## " + name + "\n" + '<center>' + table_title + '</center>\n\n' +\
-            '\n' + '&nbsp;' + '\n'+ register_data
+        #table_title = 'Register:' + name + ' ('  + addr + ')'
+        reg_addr_msg = 'address : '  + addr
+        reg_hex_reset = 'reset_value : ' + hex_reset
+        register_data = "# " + name + "\n" + reg_addr_msg + '<br>' + \
+                reg_hex_reset + '\n' + register_data
         if self.output_md:
-            os.system("mv " + png_name + " " + self.markdownpath)
-            self.data += register_data +'\n' + '<div STYLE="page-break-after: always;"></div>' + '\n'
+            os.system("mv " + svg_name + " " + self.markdownpath)
+            self.data += register_data +'\n'
         if self.output_gitbook:
-            os.system("mv " + png_name + " " + self.gitbookpath)
-            register_data += '\n' + '<div STYLE="page-break-after: always;"></div>' + '\n'
+            os.system("mv " + svg_name + " " + self.gitbookpath)
+            register_data += '\n<br>\n'
             file_path = os.path.join(self.gitbookpath, name + '.md')
             with open(file_path, 'w') as fd:
                 fd.write(register_data)
+            self.reg_data += '{% include "./' + name +  '.md" %}\n'
     
-    def add_description(self, name, des):
+    def add_description(self, name, des, access):
+        des = des.replace("；", ";")
         dess = des.split(";")
         msg = ''
         if len(dess) == 1:
-            line = "**" + name.upper() + "**:&emsp;" + des.strip(" ")
+            line = "**" + name.upper() + "** **["+ access + "]** :&emsp;" + des.strip(" ")
         else:
             for i in dess:
                 msg += "<br>&emsp;" + i.strip(" ")
-            line = "**" + name.upper() + "**:" + msg
+            line = "**" + name.upper() + "** **[" + access + "]** :" + msg
         return line
 
+    def check_file(self, file_name):
+        if self.dir_type != "link":
+            return
+        if os.path.exists(os.path.join(self.link_path, "_json_output", file_name)):
+            add_cmd = "cat " + os.path.join(self.gitbookpath, file_name) + " >> " \
+                    + os.path.join(self.link_path, "_json_output", file_name)
+            os.system(add_cmd)
+        else:
+            mv_cmd = "mv " + os.path.join(self.gitbookpath, file_name) + " " \
+                    + os.path.join(self.link_path, "_json_output", file_name)
+            os.system(mv_cmd)
+
+    def relist(self, in_list):
+        new_list = []
+        list_len = len(in_list)
+        for i in range(list_len):
+            for j in range(i, list_len):
+                #print(in_list[j])
+                if int(in_list[i]['addr'], 16) >= int(in_list[j]['addr'], 16):
+                    in_list[i], in_list[j] = in_list[j], in_list[i] 
+
+    def create_all_reg_data(self, lm):
+        for i in self.all_reg_list:
+            self.all_reg_data += lm.md_create_body_line([i['name'], i['addr'], \
+                    i['reset'], i['des']], self.output_md)
+
     def main(self):
         file_msg = os.path.basename(self.js_file).split(".json")[0]
         pp = preprocess.PreProcess()
         self.js_dict = pp.start(self.js_file)
         self.js_dpath = self.get_dpath(self.js_dict)
         lm = libmd.LibMarkdown()
-        self.all_reg_data = lm.md_create_title_line(['名称', '地址', '描述'])
+        self.all_reg_data = lm.md_create_title_line(['名称', '地址', '初值', '描述'])
         if self.output_gitbook:
-            self.gitbookpath = "./_json_output/gitbook/" + file_msg
-            if not os.path.exists(self.gitbookpath):
-                os.system("mkdir -p " + self.gitbookpath)
-            readme = os.path.join(self.gitbookpath, 'README.md')
-            summery = os.path.join(self.gitbookpath, 'SUMMARY.md')
-            if not os.path.exists(readme):
-                with open(readme, 'w') as fd:
-                    fd.write("   ")
-            with open(summery, 'w') as fd:
-                fd.write('# Summary \n\n')
-                fd.write('* [寄存器介绍](all_register.md)\n')
-                for i in range(int(len(self.js_dpath))):
-                    name = self.get_delem(self.js_dict, self.js_dpath[i], key = 'RegName' )
-                    name = name.upper()
-                    if name.lower() != "reserved":
-                        dpath = self.js_dpath[i]
-                        if dpath.find("group_reg") >= 0:
-                            is_group = 1
-                            #group_list = self.get_group_path(dpath)
-                            #for g in group_list:
-                            #    gname = self.get_delem(self.js_dict, g, key="RegName")
-                            #    gname = gname.upper()
-                            #    name = gname + "@" + name
-                        fd.write('  * [' + name + ']('+ name + '.md)\n')
-                        self.create_md3(self.js_dpath[i], lm)
-                with open(os.path.join(self.gitbookpath, 'all_register.md'), 'w') as fc:
-                    fc.write("## 寄存器列表\n" + self.all_reg_data)
-            print("\n\tcreate " + self.gitbookpath +" success") 
+            if self.dir_type == "alone":
+                self.gitbookpath = os.path.join("_json_output/gitbook/", file_msg)
+                if not os.path.exists(self.gitbookpath):
+                    os.system("mkdir -p " + self.gitbookpath)
+                readme = os.path.join(self.gitbookpath, 'README.md')
+                summary = os.path.join(self.gitbookpath, 'SUMMARY.md')
+                if not os.path.exists(readme):
+                    with open(readme, 'w') as fd:
+                        fd.write("   ")
+                with open(summary, 'w') as fd:
+                    fd.write('# Summary \n\n')
+                    fd.write('* [' + file_msg + '寄存器列表](all_register.md)\n')
+                    for i in self.js_dpath:
+                        self.path_addr_dict[i] = self.get_addr(i)
+                    self.max_len = len(self.path_addr_dict[self.js_dpath[-1]][2:])
+                    for i in range(int(len(self.js_dpath))):
+                        name = self.get_delem(self.js_dict, self.js_dpath[i], key = 'RegName' )
+                        name = name.upper()
+                        if name != "RESERVED":
+                            dpath = self.js_dpath[i]
+                            fd.write('  * [' + name + ']('+ name + '.md)\n')
+                            self.create_md3(self.js_dpath[i], lm)
+                    with open(os.path.join(self.gitbookpath, 'all_register.md'), 'w') as fc:
+                        fc.write("## " + file_msg + "寄存器列表\n" + self.all_reg_data)
+                print("\n\tcreate " + self.gitbookpath +" success") 
+            elif self.dir_type == "link":
+                self.gitbookpath = os.path.join(self.link_path, "_json_output/", file_msg)
+                path_msg = "_json_output/" + str(file_msg)
+                if not os.path.exists(self.gitbookpath):
+                    os.system("mkdir -p " + self.gitbookpath)
+                readme = os.path.join(self.gitbookpath, 'README.md')
+                summary = os.path.join(self.gitbookpath, 'SUMMARY.md')
+                if not os.path.exists(readme):
+                    with open(readme, 'w') as fd:
+                        fd.write("   ")
+                with open(summary, 'w') as fd:
+                    fd.write('# Summary \n\n')
+                    reg_list_path = os.path.join(path_msg, "all_register.md") 
+                    reg_detail_path = os.path.join(path_msg, "detail.md") 
+                    fd.write('* [' + file_msg + '寄存器列表](' + reg_list_path + ')\n')
+                    fd.write('* [寄存器](' + reg_detail_path + ')\n')
+                    for i in self.js_dpath:
+                        self.path_addr_dict[i] = self.get_addr(i)
+                    self.max_len = len(self.path_addr_dict[self.js_dpath[-1]][2:])
+                    for i in range(int(len(self.js_dpath))):
+                        name = self.get_delem(self.js_dict, self.js_dpath[i], key = 'RegName' )
+                        name = name.upper()
+                        if name != "RESERVED":
+                            dpath = self.js_dpath[i]
+                            #md_path = os.path.join(path_msg, name + ".md")
+                            #fd.write('  * [' + name + ']('+ md_path + ')\n')
+                            self.create_md3(self.js_dpath[i], lm)
+                    self.relist(self.all_reg_list)
+                    self.create_all_reg_data(lm)
+                    with open(os.path.join(self.gitbookpath, 'all_register.md'), 'w') as fc:
+                        fc.write("## " + file_msg + "寄存器列表\n" + self.all_reg_data)
+                    with open(os.path.join(self.gitbookpath, 'detail.md'), 'w') as fc:
+                        fc.write(self.reg_data)
+                self.check_file('README.md')
+                self.check_file('SUMMARY.md')
+                print("\n\tcreate " + self.gitbookpath +" success") 
         if self.output_md:
-            self.markdownpath = "./_json_output/" + file_msg
+            #self.markdownpath = "./_json_output/" + file_msg
+            self.markdownpath = os.path.join("./_json_output/markdwon/", file_msg)
             if not os.path.exists(self.markdownpath):
                 os.system("mkdir -p " + self.markdownpath)
-            md_file = os.path.join(self.markdownpath, \
-                    os.path.basename(self.js_file).split(".json")[0] + ".md")
+            md_file = os.path.join(self.markdownpath, file_msg + ".md")
+            for i in self.js_dpath:
+                selfgpath_addr_dict[i] = self.get_addr(i)
+            self.max_len = len(self.path_addr_dict[self.js_dpath[-1]][2:])
             for i in self.js_dpath:
                 self.create_md3(i, lm)
-            self.data = "## 寄存器列表\n" + self.all_reg_data + self.data
+            self.data = "# 寄存器列表\n" + self.all_reg_data + "\n"\
+                    + "# 寄存器详情\n" + self.data.replace("#", "##")
             with open(md_file, 'w') as fd:
                 fd.write(self.data)
             print("\n\tcreate " + md_file + " success") 
@@ -429,13 +535,14 @@ class TranslateJson(object):
 def usage():
     """
     Usage:
-        jsontomd -f json_file (-g)
+        jsontomd -f json_file (-g) (-l remote_dir)
       exp
-        jsontomd -f npu_register.json
+        jsontomd -f npu_register.json -g -l target_dir
 
     parameters:
         -f : 指定json文件
         -g : 加-g ,生成gitbook目录, 不加-g 生成单个markdown文件
+        -l : 指定链接目录, 把生成的gitbook目录,放到指定目录下
     """
     print(usage.__doc__)
 
@@ -443,13 +550,20 @@ def main():
     if len(sys.argv) <= 1:
             usage()
             sys.exit()
-    opts,args = getopt(sys.argv[1:],"f:gh")
+    opts,args = getopt(sys.argv[1:],"f:l:gh")
     js_file = "demo.json"
+    link_path = ''
     output_md = 1
     output_gitbook = 0
+    dir_type = "alone"
     for k,v in opts:
         if k == "-f":
             js_file = v
+        if k == "-l":
+            link_path = v
+            dir_type = "link"
+            output_gitbook = 1
+            output_md = 0
         if k == "-g":
             output_gitbook = 1
             output_md = 0
@@ -457,7 +571,7 @@ def main():
             usage()
             sys.exit()
     #print("opts:", opts, "args", args)
-    tj = TranslateJson(js_file, output_md, output_gitbook)
+    tj = TranslateJson(js_file, link_path, dir_type, output_md, output_gitbook)
     tj.main()
 
 if __name__ == "__main__":
